/*
 * FileWatcher.java - Part of the FileWatcher plugin for jEdit.
 * Copyright (C) 1999 Ollie Rutherfurd
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


import org.gjt.sp.jedit.*;
import org.gjt.sp.jedit.msg.*;
import org.gjt.sp.util.Log;

import java.io.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;

/**
*	@author Ollie Rutherfurd <a href="mailto:oliver@rutherfurd.net">Ollie Rutherfurd</a>
*/
public class FileWatcher implements EBComponent, ChangeListener{

	private View view;
	private WatchList watchList;
	private CurrentBufferWatcher currentBufferWatcher;
	private boolean watchCurrentBuffer = false;

	private int reloadAction;

	private boolean beepOnAutoReload;

	private static final int RELOAD_PROMPT = 0;
	private static final int RELOAD_AUTO = 1;
	private static final int RELOAD_NOTHING = 2;

	public FileWatcher(View view, boolean watchCurrentBuffer){

		reloadAction = RELOAD_PROMPT;
		beepOnAutoReload = true;

		Buffer[] buffers = jEdit.getBuffers();
		this.view = view;
		watchList = new WatchList();
		currentBufferWatcher = new CurrentBufferWatcher();

		for(int i = 0; i < buffers.length; i++){
			Log.log(Log.DEBUG, this, "Adding -> " + buffers[i].getPath());
			watchList.add(buffers[i].getPath());
		}

		setWatchCurrentBuffer(watchCurrentBuffer);
	}


	/**
	* 
	*/
	public FileWatcher(View view){
		this(view, false);
	}


	/**
	* Sets whether to watch the current buffer or not.
	*/
	public synchronized void setWatchCurrentBuffer(boolean watchCurrentBuffer){
		if(this.watchCurrentBuffer != watchCurrentBuffer){
			this.watchCurrentBuffer = watchCurrentBuffer;
			currentBufferWatcher.setKeepAlive(watchCurrentBuffer);
		}
	}


	/**
	* Sets the reload action to take.
	*/
	public synchronized void setReloadAction(int reloadAction){
		this.reloadAction = reloadAction;
	}


	/**
	*
	*/
	public synchronized void setBeepOnAutoReload(boolean beepOnAutoReload){
		this.beepOnAutoReload = beepOnAutoReload;
	}


	/**
	* returns an integer constant based on the reloadAction string
	*/
	public static int translateReloadAction(String reloadAction){
		if(reloadAction.equals("auto-reload"))
			return RELOAD_AUTO;
		if(reloadAction.equals("nothing"))
			return RELOAD_NOTHING;

		return RELOAD_PROMPT;
	}

	/**
    * Initializes and starts listening for events.
    */
   public synchronized void start(){
		Log.log(Log.DEBUG, this, "Starting");
		// System.out.println("FileWatcher.start");	// ##
		EditBus.addToBus(this);
	}


	/**
    * Stops listening for events.
    */
   public synchronized void stop(){
		Log.log(Log.DEBUG, this, "Stopping");
		// System.out.println("FileWatcher.stop");	// ##
		currentBufferWatcher.keepAlive = false;
      EditBus.removeFromBus(this);
   }


	/**
    * EditBus message handling.
    */
   public void handleMessage(EBMessage msg) {
		if(msg instanceof BufferUpdate){
			BufferUpdate bu = (BufferUpdate) msg;
			Buffer buf = bu.getBuffer();

			if(bu.getWhat() == BufferUpdate.CREATED){
				addCurrentFile(buf);
			}
			else if(bu.getWhat() == BufferUpdate.CLOSED){
				removeCurrentFile(buf);
			}
			else if(bu.getWhat() == BufferUpdate.DIRTY_CHANGED){
				updateFileStamp(buf);
			}
			else if(bu.getWhat() == BufferUpdate.LOADING){
				updateFileStamp(buf);
			}
		}
		else if(msg instanceof ViewUpdate){
			ViewUpdate vu = (ViewUpdate) msg;
			if(vu.getView() == view){
				if(vu.getWhat() == ViewUpdate.BUFFER_CHANGED){
					try{
						if(vu.getView().getBuffer().isNewFile() == false && watchList.getTime(vu.getView().getBuffer().getPath()) < (new File(vu.getView().getBuffer().getPath())).lastModified()){
							Log.log(Log.DEBUG, this, "Need to reload -> " + view.getBuffer().getPath());
							handleReload(view, view.getBuffer().getPath());
						}
					}catch(Exception e){Log.log(Log.ERROR, this, "FileWatcher: error trying to evaluate whether to reload file (" + vu.getView().getBuffer().getPath() + ") -> " + e);}
				}
			}
		}
   }


	/**
	* Add the currently selected buffer to the file list.
	*/
	public synchronized void addCurrentFile(Buffer buf){
		// don't add untitled buffers
		try{
			if(! buf.isNewFile() )
				Log.log(Log.DEBUG, this, "Adding -> " + buf.getPath());
				watchList.add(buf.getPath());
		}catch(Exception e){ Log.log(Log.ERROR, this, "FileWatcher: trying to add (" + buf.getPath() + ") -> " + e); }
	}


	/**
	* Removes the currently selected buffer from the watch list.
	*/
	public synchronized void removeCurrentFile(Buffer buf){
		try{
			Log.log(Log.DEBUG, this, "Removing -> " + buf.getPath());
			watchList.remove(buf.getPath());
		}catch(Exception e){ Log.log(Log.ERROR, this, "FileWatcher: trying to remove (" + buf.getPath() + ") -> " + e); }
	}


	/**
	* Updates the current file's timestamp from the file saved on disk
	*/
	public synchronized void updateFileStamp(Buffer buf){
		try{
			Log.log(Log.DEBUG, this, "Updating timestamp -> " + buf.getPath());
			watchList.updateTimeFromFile(buf.getPath());
		}catch(Exception e){ Log.log(Log.ERROR, this, "FileWatcher: trying to watchList.updateTimeFromFile(" + buf.getPath() + ") -> " + e); }
	}


	/**
	* Handles the reloading of a buffer from disk.
	*/
	public synchronized void handleReload(View view, String filename){
		if(reloadAction == RELOAD_NOTHING){
			watchList.updateTimeFromFile(filename);
			Log.log(Log.NOTICE, this, "No reload action taken, updated timestamp -> " + filename);
		}
		else if(reloadAction == RELOAD_AUTO){
			Log.log(Log.NOTICE, this ,"Auto-Reloading -> " + filename);
			view.saveCaretInfo();
			view.getBuffer().load(view);
			view.loadCaretInfo();
			view.focusOnTextArea();

			if(beepOnAutoReload)
				Toolkit.getDefaultToolkit().beep();
		}
		else
			SwingUtilities.invokeLater(new PromptForFileReload(view, filename, watchList));
	}


	/**
	*
	*/
	public synchronized void stateChanged(ChangeEvent evt) {}

	/**
	* Thread to check whether the current buffer is outdated.
	*/
	private class CurrentBufferWatcher extends Thread{

		/** How long to sleep the thread for while waiting for things to happen */
		private static final int SLEEP_INTERVAL = 1000;


		/** when to stop thread */
		boolean keepAlive = true;


		/**
		* Starts or or sets the flag to stop the thread watching the current buffer
		*/
		void setKeepAlive(boolean keepAlive){
			if(keepAlive == true) this.start();
			this.keepAlive = keepAlive;
		}


		/**
		* Constructor
		*/
		public CurrentBufferWatcher(){}


		/**
		* Every <code>SLEEP_INTERVAL</code> the thread checks whether the current buffer needs to be reloaded.
		*/
		public void run(){

			try{
				Log.log(Log.DEBUG, this, "FileWatcher.CurrentBufferWatcher starting run()");
			}catch(Exception e){ System.err.println("Error logging CurrentBufferWatcher starting: " + e); }

			while(keepAlive){

				try{
					if(view.getBuffer().isNewFile() == false && watchList.getTime(view.getBuffer().getPath()) < watchList.getLastModifiedTime(view.getBuffer().getPath())){
						handleReload(view, view.getBuffer().getPath());
					}
				}catch(Exception e){
					Log.log(Log.ERROR, this, "FileWatcher.CurrentBufferWatcher:  checking whether buffer needs to be reloaded. -> " + e);
				}

				try{
					sleep(SLEEP_INTERVAL);
				}catch(Exception e){
					Log.log(Log.ERROR, this, "FileWatcher.CurrentBufferWatcher:  trying to sleep CurrentBufferWatcher thread. -> " + e);
				}
			}

			try{
				Log.log(Log.DEBUG, this, "FileWatcher.CurrentBufferWatcher ending run()");
			}catch(Exception e){ System.err.println("Error logging CurrentBufferWatcher terminating: " + e); }
		}

	}

}
