/*
 * FileWatcherPlugin.java - Part of the FileWatcher plugin for jEdit.
 * Copyright (C) 1999 Ollie Rutherfurd
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
import org.gjt.sp.jedit.*;
import org.gjt.sp.jedit.gui.*;
import org.gjt.sp.jedit.msg.*;
import org.gjt.sp.util.Log;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;

/* ------------------------------------------------------------------------------------------------
	TODO:
		- add support to handle open buffers being 're-loaded' if opened with '-reuse' parameter
*/


/**
*	@author Ollie Rutherfurd <a href="mailto:oliver@rutherfurd.net">Ollie Rutherfurd</a>
*/
public class FileWatcherPlugin extends EBPlugin{

	/** Hash for managing FileWatchers (one per view) */
	private static Hashtable watchersMap = new Hashtable();


	/**
   * Called at start up time when the plugin is being loaded.
   */
	public void start() {
		Log.log(Log.DEBUG, this, "Starting");
		super.start();
	}


	/**
	* Called when jEdit is exiting.
	*/
	public void stop() {
		Log.log(Log.DEBUG, this, "Stopping");
		super.stop();
	}

  
	/**
	* This allows plugins to add their own option pane to the
	* <code>OptionsDialog</code>.
	*/
  	public void createOptionPanes(OptionsDialog od) {
		od.addOptionPane(new FileWatcherOptionPane());
	}


 	public void createMenuItems(View view, Vector menus, Vector menuItems) {}


	/**
	* Handle EditBus messages.
	*/
	public void handleMessage(EBMessage msg){
		if(msg instanceof ViewUpdate) {
			ViewUpdate vu = (ViewUpdate) msg;

			if(vu.getWhat() == ViewUpdate.CREATED) {
				attachWatcherToView(vu.getView());
			}
			else if(vu.getWhat() == ViewUpdate.CLOSED){
				detachWatcherFromView(vu.getView());
			}
		}
		else if(msg instanceof PropertiesChanged) {
			propertiesChanged();
		}
	}

	/**
  	* Updates FileWatch properties.
  	*/
	void propertiesChanged(){
		String reloadAction = FileWatcherOptionPane.getReloadActionProperty("filewatcher.reload-action", "prompt");
		boolean beepOnAutoReload = FileWatcherOptionPane.getBooleanProperty("filewatcher.beep-on-auto-reload", true);
		boolean watchCurrentBuffer = FileWatcherOptionPane.getBooleanProperty("filewatcher.watch-current-buffer", false);
		
		Log.log(Log.DEBUG, this, "propertiesChanged[reloadAction=" + reloadAction + "; beepOnAutoLoad=" + beepOnAutoReload + "; watchCurrentBuffer=" + watchCurrentBuffer + "]");
		// System.out.println("FileWatcherPlugin.propertiesChanged; reloadAction=" + reloadAction + "; beepOnAutoLoad=" + beepOnAutoReload);	// ##
		
		View[] views = jEdit.getViews();
		
		for(int i = 0; i < views.length; i++){
			FileWatcher fw = (FileWatcher) watchersMap.get(views[i]);
			if(fw != null){
				fw.setBeepOnAutoReload(beepOnAutoReload);
				fw.setReloadAction(FileWatcher.translateReloadAction(reloadAction));
				fw.setWatchCurrentBuffer(watchCurrentBuffer);
			}
		}
	}
  
  
  	/**
  	* Adds a FileWatcher to <code>view</code>
  	*/
	private void attachWatcherToView(View view){

		Log.log(Log.DEBUG, this, "attaching to view");
		// System.out.println("FileWatcherPlugin.attachWatcherToView: attaching FileWatcher to view..."); // ##

		String reloadAction = FileWatcherOptionPane.getReloadActionProperty("filewatcher.reload-action", "prompt");
		boolean beepOnAutoReload = FileWatcherOptionPane.getBooleanProperty("filewatcher.beep-on-auto-reload", true);
		boolean watchCurrentBuffer = FileWatcherOptionPane.getBooleanProperty("filewatcher.watch-current-buffer", false);

		FileWatcher fileWatcher = new FileWatcher(view, watchCurrentBuffer);

		fileWatcher.setBeepOnAutoReload(beepOnAutoReload);
		fileWatcher.setReloadAction(FileWatcher.translateReloadAction(reloadAction));

		fileWatcher.start();
		watchersMap.put(view, fileWatcher);

	}


	/**
  	* Removes the FileWatcher attached to <code>view</code>
  	*/
	private void detachWatcherFromView(View view){
		System.out.println("FileWatcherPlugin.detachWatcherFromView: detaching FileWatcher to view..."); // ##

		try{
			watchersMap.remove(view);
		}catch(Exception e){ Log.log(Log.ERROR, this, "FileWatcherPlugin.detachWatcherFromView -> " + e); }
	}

}
